/*
 * Copyright (c) 2023 - 2024. KeyWare.Co.Ltd All rights reserved.
 * 项目名称：C++ 信息安全性设计准则
 * 项目描述：用于检查C++源代码的安全性设计准则的Sonarqube插件
 * 版权说明：本软件属北京关键科技股份有限公司所有，在未获得北京关键科技股份有限公司正式授权情况下，任何企业和个人，不能获取、阅读、安装、传播本软件涉及的任何受知识产权保护的内容。
 */
package com.keyware.sonar.cxx;

import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.RegisterExtension;
import org.junit.jupiter.api.io.TempDir;
import org.sonar.api.batch.sensor.internal.SensorContextTester;
import org.sonar.api.config.internal.MapSettings;
import org.sonar.api.utils.log.LogTesterJUnit5;
import org.sonar.api.utils.log.LoggerLevel;

import java.io.File;
import java.util.ArrayList;
import java.util.List;

import static org.assertj.core.api.Assertions.assertThat;

class DroppedPropertiesSensorTest {

  @TempDir
  File tempDir;

  @RegisterExtension
  public LogTesterJUnit5 logTester = new LogTesterJUnit5();

  @Test
  void testNoMsg() throws Exception {
    var contextTester = SensorContextTester.create(tempDir);
    var mapSettings = new MapSettings().setProperty("sonar.cxx.xxx", "value");
    contextTester.setSettings(mapSettings);
    List<String> analysisWarnings = new ArrayList<>();
    var sensor = new DroppedPropertiesSensor(analysisWarnings::add);
    sensor.execute(contextTester);

    assertThat(logTester.logs(LoggerLevel.WARN)).isEmpty();
    assertThat(analysisWarnings).isEmpty();
  }

  @Test
  void testNoLongerSupported() throws Exception {
    var contextTester = SensorContextTester.create(tempDir);
    var mapSettings = new MapSettings().setProperty("sonar.cxx.cppncss.reportPaths", "value");
    contextTester.setSettings(mapSettings);
    List<String> analysisWarnings = new ArrayList<>();
    var sensor = new DroppedPropertiesSensor(analysisWarnings::add);
    sensor.execute(contextTester);

    var msg = "CXX property 'sonar.cxx.cppncss.reportPaths' is no longer supported.";
    assertThat(logTester.logs(LoggerLevel.WARN)).contains(msg);
    assertThat(analysisWarnings).containsExactly(msg);
  }

  @Test
  void testNoLongerSupportedWithInfo() throws Exception {
    var contextTester = SensorContextTester.create(tempDir);
    var mapSettings = new MapSettings().setProperty("sonar.cxx.suffixes.sources", "value");
    contextTester.setSettings(mapSettings);
    List<String> analysisWarnings = new ArrayList<>();
    var sensor = new DroppedPropertiesSensor(analysisWarnings::add);
    sensor.execute(contextTester);

    var msg = "CXX property 'sonar.cxx.suffixes.sources' is no longer supported."
            + " Use key 'sonar.cxx.file.suffixes' instead.";
    assertThat(logTester.logs(LoggerLevel.WARN)).contains(msg);
    assertThat(analysisWarnings).containsExactly(msg);
  }

}
